@@ -3,7 +3,6 @@ use std::{
     time::Duration,
 };
 
-use bytes::Bytes;
 use exocore_core::{cell::Node, time::Instant};
 use futures::task::{Context, Poll};
 use libp2p::{
@@ -14,7 +13,7 @@ use libp2p::{
     },
 };
 
-use super::protocol::{ExocoreProtoHandler, ExocoreProtoMessage};
+use super::protocol::{ExocoreProtoHandler, MessageData};
 
 const MAX_PEER_QUEUE: usize = 20;
 const DEFAULT_DIALING_MESSAGE_TIMEOUT: Duration = Duration::from_secs(5);
@@ -34,7 +33,7 @@ pub struct ExocoreBehaviour {
     last_redial_check: Option<Instant>,
 }
 
-type BehaviourAction = NetworkBehaviourAction<ExocoreProtoMessage, ExocoreBehaviourEvent>;
+type BehaviourAction = NetworkBehaviourAction<MessageData, ExocoreBehaviourEvent>;
 
 impl ExocoreBehaviour {
     pub fn send_message(
@@ -42,7 +41,7 @@ impl ExocoreBehaviour {
         peer_id: PeerId,
         expiration: Option<Instant>,
         connection: Option<ConnectionId>,
-        data: Bytes,
+        msg: MessageData,
     ) {
         let handler = if let Some(connection_id) = connection {
             NotifyHandler::One(connection_id)
@@ -55,7 +54,7 @@ impl ExocoreBehaviour {
                 let event = NetworkBehaviourAction::NotifyHandler {
                     peer_id,
                     handler,
-                    event: ExocoreProtoMessage { data },
+                    event: msg,
                 };
 
                 self.actions.push_back(event);
@@ -70,7 +69,7 @@ impl ExocoreBehaviour {
                     event: NetworkBehaviourAction::NotifyHandler {
                         peer_id,
                         handler,
-                        event: ExocoreProtoMessage { data },
+                        event: msg,
                     },
                     expiration: Some(expiration),
                 });
@@ -221,12 +220,7 @@ impl NetworkBehaviour for ExocoreBehaviour {
         }
     }
 
-    fn inject_event(
-        &mut self,
-        peer_id: PeerId,
-        connection: ConnectionId,
-        msg: ExocoreProtoMessage,
-    ) {
+    fn inject_event(&mut self, peer_id: PeerId, connection: ConnectionId, message: MessageData) {
         if let Some(peer) = self.peers.get_mut(&peer_id) {
             trace!("Received message from {}", peer.node);
 
@@ -235,7 +229,7 @@ impl NetworkBehaviour for ExocoreBehaviour {
                     ExocoreBehaviourEvent::Message(ExocoreBehaviourMessage {
                         source: peer_id,
                         connection,
-                        data: msg.data,
+                        message,
                     }),
                 ));
         }
@@ -268,7 +262,7 @@ impl NetworkBehaviour for ExocoreBehaviour {
         &mut self,
         _ctx: &mut Context,
         _params: &mut impl PollParameters,
-    ) -> Poll<NetworkBehaviourAction<ExocoreProtoMessage, ExocoreBehaviourEvent>> {
+    ) -> Poll<NetworkBehaviourAction<MessageData, ExocoreBehaviourEvent>> {
         // check if we could try to dial to disconnected nodes
         let redial_check = self
             .last_redial_check
@@ -339,11 +333,10 @@ pub enum ExocoreBehaviourEvent {
     PeerStatus(PeerId, PeerStatus),
 }
 
-#[derive(Debug, Clone, PartialEq, Eq, Hash)]
 pub struct ExocoreBehaviourMessage {
     pub source: PeerId,
     pub connection: ConnectionId,
-    pub data: Bytes,
+    pub message: MessageData,
 }
 
 #[derive(Debug, Copy, Clone, PartialEq)]
